.DATA
ALIGN 16
;
; ici, ce sont les tmoins concernant les informations du microprocesseur
;
_Temoins_Aptitudes_Du_MicroProcesseur_	DWORD ?


.CODE
ALIGN 16
;
; obtenir des informations sur le microprocesseur qui utilise le programme
; bit 0 = Cpuid support ? 1=oui, 0=non
; bit 1 = Fpu support ? 1=oui, 0=non
; bit 2 = Rdtsc support ? 1=oui, 0=non
; bit 3 = cMov support ? 1=oui, 0=non
; bit 4 = Mmx support ? 1=oui, 0=non
; bit 5 = Sse support ? 1=oui, 0=non
; bit 6 = Sse2 support ? 1=oui, 0=non
; bit 7 = Sse3 support ? 1=oui, 0=non
;
; Syntaxe :
; call AptitudesDuProcesseur
;
; Retourne :
; eax et la variable _Temoins_Aptitudes_Du_MicroProcesseur_ sont dfinis avec les bits positionns
;
AptitudesDuProcesseur PROC
		push ebx										;; empiler ebx
		push ecx										;; empiler ecx
		push edx										;; empiler edx
		push esi										;; empiler esi

		xor esi,esi									;; mettre esi (le registre qui stockera les bits)  0
; l'instruction CPUID est supporte, seulement s'il est possible de positionner/enlever le flag ID (EFLAGS bit 21).
		pushfd										;; ) copier les flags+flags etendus dans eax et edx
		pop eax										;; )
		mov edx,eax									;; )
		btc eax,21									;; inverser le flag ID (bit 21)
		push eax										;; ) et recopier le rsultat dans les flags
		popfd										;; )
		pushfd										;;	) puis on recopie  nouveau les flags dans eax
		pop eax										;;	)
		cmp eax,edx									;; comparer le rsultat  notre prcdente sauvegarde
		je PasSupportCpuid								;; si c'est semblable, c'est que notre modif n'a eu aucun effet, alors aller PasSupportCpuid
		bts esi,0										;; positionner le bit correspondant dans esi

; ici, l'instruction cpuid est supporte, on va donc s'en servir pour connaitre les aptitudes du processeur
;		mov eax,0										;; ici, on dtermine le type d'infos que l'on veut rcuprer
;		cpuid										;; fixer les registres eax,ebx,ecx et edx avec les informations du cpu
;													;; (eax contient le nombre d'entres cpuid maximales)

; ici, on passe aux informations suivantes
		mov eax,1										;; ici, on dtermine les infos les plus importantes (du moins pour la programmation)
		cpuid										;; fixer les registres eax,ebx,ecx et edx avec les informations du cpu
													;; (eax contient le type, le model,la famille,...)
													;; (edx contient les caractristiques disponibles)
; les instructions fpu sont disponibles ?
		bt edx,0										;; tester le bit 0
		jnc PasSupportFpu								;; si le flag carry n'est pas positionn, aller PasSupportFpu
		bts esi,1										;; positionner le bit correspondant dans esi
PasSupportFpu:
; l'instruction rdtsc est disponible ?
		bt edx,4										;; tester le bit 4
		jnc PasSupportRdtsc								;; si le flag carry n'est pas positionn, aller PasSupportRdtsc
		bts esi,2										;; positionner le bit correspondant dans esi
PasSupportRdtsc:
; les instructions cmov sont disponibles ?
		bt edx,15										;; tester le bit 15
		jnc PasSupportCmov								;; si le flag carry n'est pas positionn, aller PasSupportCmov
		bts esi,3										;; positionner le bit correspondant dans esi
PasSupportCmov:
; les instructions mmx sont disponibles ?
		bt edx,23										;; tester le bit 23
		jnc PasSupportMmx								;; si le flag carry n'est pas positionn, aller PasSupportMmx
		bts esi,4										;; positionner le bit correspondant dans esi
PasSupportMmx:
; les instructions sse sont disponibles ?
		bt edx,25										;; tester le bit 25
		jnc PasSupportSse								;; si le flag carry n'est pas positionn, aller PasSupportSse
		bts esi,5										;; positionner le bit correspondant dans esi
PasSupportSse:
; les instructions sse2 sont disponibles ?
		bt edx,26										;; tester le bit 26
		jnc PasSupportSse2								;; si le flag carry n'est pas positionn, aller PasSupportSse2
		bts esi,6										;; positionner le bit correspondant dans esi
PasSupportSse2:
; les instructions sse3 sont disponibles ?
		bt ecx,0										;; tester le bit 0
		jnc PasSupportSse3								;; si le flag carry n'est pas positionn, aller PasSupportSse3
		bts esi,7										;; positionner le bit correspondant dans esi
PasSupportSse3:
;; les instructions Xxx sont disponibles ?
;		bt edx,Xxx									;; tester le bit 
;		jnc PasSupportXxx								;; si le flag carry n'est pas positionn, aller
;		bts esi,x										;; positionner le bit correspondant dans esi
;PasSupportXxx:

; ici, on passe aux informations suivantes
;		mov eax,2										;; ici, on dtermine les infos concernant le cache
;		cpuid										;; fixer les registres eax,ebx,ecx et edx avec les informations du cpu
PasSupportCpuid:
		mov _Temoins_Aptitudes_Du_MicroProcesseur_,esi		;; sauvegarder les tmoins dans notre variable
		mov eax,esi									;; et les recopier dans eax

		pop esi										;; dsempiler esi
		pop edx										;; dsempiler edx
		pop ecx										;; dsempiler ecx
		pop ebx										;; dsempiler ebx
	ret												;; retourner, sortir de la procdure
AptitudesDuProcesseur ENDP


ALIGN 16
;
; effacer un bloc memoire (en le remplissant de 0)
;
; Syntaxe :
; mov eax,{taille  ffacer en octets}
; mov edi,{adresse du bloc de destination}
; call EffacerBlocMemoire
;
; Retourne :
; rien
;
EffacerBlocMemoire PROC
		push ecx										;; empiler ecx
		push edx										;; empiler edx

		cld											;; dfinir le flag de direction
; ici on traite les dwords
		mov ecx,eax									;; copier eax (la taille) dans ecx et edx
		mov edx,eax									;; copier eax (la taille) dans edx
		xor eax,eax									;; effacer eax
		shr ecx,2										;; diviser ecx par 4, pour ne conserver que le nombre de dwords
		rep stosd										;; transferer le contenu de eax  l'adresse en edi
; ici on traite les octets
		mov ecx,edx									;; copier edx dans ecx
		and ecx,00000000000000000000000000000011b			;; ne conserver que les bits correspondants aux octets en ecx
		jnz Label1									;; si c'est diffrent de 0, aller Label1
		sub edi,edx									;; soustraire la taille  edi pour retrouver (l'adresse d'origine)
		mov eax,edx									;; replacer la taille dans eax

		pop edx										;; dsempiler edx
		pop ecx										;; dsempiler ecx
	ret												;; retourner

Label1:												;; (rep ne doit pas se trouver sur la mme ligne qu'un label)
		rep stosb										;; transferer le contenu de al  l'adresse en edi
		sub edi,edx									;; soustraire la taille  edi pour retrouver (l'adresse d'origine)
		mov eax,edx									;; replacer la taille dans eax

		pop edx										;; dsempiler edx
		pop ecx										;; dsempiler ecx
	ret												;; retourner
EffacerBlocMemoire ENDP


; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
; *
; * routines de traitement de chaines de caractres
; *
; * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
;.DATA
ALIGN 16
_TCA_Compteur_Chaines_Temporaire_		DWORD ?


.CODE
ALIGN 16
;
; convertir un dword en texte au format dcimale (routine la plus stable pour le travail)
;
; syntaxe :
; mov eax,{la valeur  convertir en chane}
; mov esi,{adresse (offset) de la chaine de caractres  crer}
; call ConvertirValeurEnChaineDecimale
;
;
; Retourne :
; eax = la longueur de la chane cre
;
ConvertirValeurEnChaineDecimale PROC
		test eax,eax									;; fixer les flags de eax
		jz CasZero									;; si zro, aller CasZero

		push ebx										;; empiler ebx
		push ecx										;; empiler ecx
		push edx										;; empiler edx
		push esi										;; empiler esi

		mov ecx,eax									;; copier eax dans ecx
		inc eax										;; pour que la division qui suit donne le rsultat exact
		jz CasMoinsUn									;; si FFFFFFFFh+1, aller CasMoinsUn
; on va diviser eax par 100000 de manire optimise
;		mov edx,2814749768								;; placer 2814749768 (remplacement de 2814749767 et eax+1) dans edx
		mov edx,2814749767								;; placer 2814749767 dans edx
		mul edx										;; multiplier eax par 2814749768
		shr edx,16									;; dcaler edx de 16 bits  droite
		jz Label01									;; si c'est gal  0, aller Label01
		mov eax,100000									;; placer 100000 dans eax
		mov ebx,edx									;; copier edx dans ebx
		mul edx										;; multiplier edx par 100000
		sub ecx,eax									;; soustraire le rsultat en eax  ecx
		mov edx,ebx									;; replacer ebx dans edx
; sinon, on traite la partie suprieure du nombre
		mov eax,429497									;; on remplace eax par 429497
		mov ebx,10									;; placer 10 (notre multiplicateur dcimale) dans ebx
		mul edx										;; multiplier edx par 429497 (pour pouvoir extraire correctement les 5 dcimales suprieures)
		jc Label02									;; s'il existe un dpassement, aller Label02
		dec esi										;; dcrmenter l'adresse
		mul ebx										;; multiplier eax par 10
		jc Label03									;; s'il existe un dpassement, aller Label03
		dec esi										;; dcrmenter l'adresse
		mul ebx										;; multiplier eax par 10
		jc Label04									;; s'il existe un dpassement, aller Label04
		dec esi										;; dcrmenter l'adresse
		mul ebx										;; multiplier eax par 10
		jc Label05									;; s'il existe un dpassement, aller Label05
		dec esi										;; dcrmenter l'adresse
		jmp Label06									;; aller Label06
;ALIGN 4
; ici, on traite la partie infrieure du nombre
Label01:	mov eax,429497									;; on remplace eax par 429497
		mov ebx,10									;; placer 10 (notre multiplicateur dcimale) dans ebx
		sub esi,5										;; enlever 5 caractres  esi
		mul ecx										;; multiplier ecx par 429497 (pour pouvoir extraire correctement les 5 dcimales suprieures)
		jc Label07									;; s'il existe un dpassement, aller Label07
		dec esi										;; dcrmenter l'adresse
		mul ebx										;; multiplier eax par 10
		jc Label08									;; s'il existe un dpassement, aller Label08
		dec esi										;; dcrmenter l'adresse
		mul ebx										;; multiplier eax par 10
		jc Label09									;; s'il existe un dpassement, aller Label09
		dec esi										;; dcrmenter l'adresse
		mul ebx										;; multiplier eax par 10
		jc Label10									;; s'il existe un dpassement, aller Label10
		dec esi										;; dcrmenter l'adresse
		jmp Label11									;; aller Label11
;ALIGN 4
; ici, on va placer XXXXXXXXXX et le zro final
Label02:	add dl,"0"									;; ajouter le caractre de base "0"  la valeur en edx
		mov BYTE PTR [esi],dl							;; placer l'octet en dl  l'adresse en esi
		mul ebx										;; multiplier eax par 10
Label03:	add dl,"0"									;; ajouter le caractre de base "0"  la valeur en edx
		mov BYTE PTR [esi+1],dl							;; placer l'octet en dl  l'adresse en esi
		mul ebx										;; multiplier eax par 10
Label04:	add dl,"0"									;; ajouter le caractre de base "0"  la valeur en edx
		mov BYTE PTR [esi+2],dl							;; placer l'octet en dl  l'adresse en esi
		mul ebx										;; multiplier eax par 10
Label05:	add dl,"0"									;; ajouter le caractre de base "0"  la valeur en edx
		mov BYTE PTR [esi+3],dl							;; placer l'octet en dl  l'adresse en esi
Label06:	mul ebx										;; multiplier eax par 10
		add dl,"0"									;; ajouter le caractre de base "0"  la valeur en edx
		mov BYTE PTR [esi+4],dl							;; placer l'octet en dl  l'adresse en esi
		mov eax,429497									;; on remplace eax par 429497
		mul ecx										;; multiplier ecx par 429497 (pour pouvoir extraire correctement les 5 dcimales suprieures)
Label07:	add dl,"0"									;; ajouter le caractre de base "0"  la valeur en edx
		mov BYTE PTR [esi+5],dl							;; placer l'octet en dl  l'adresse en esi
		mul ebx										;; multiplier eax par 10
Label08:	add dl,"0"									;; ajouter le caractre de base "0"  la valeur en edx
		mov BYTE PTR [esi+6],dl							;; placer l'octet en dl  l'adresse en esi
		mul ebx										;; multiplier eax par 10
Label09:	add dl,"0"									;; ajouter le caractre de base "0"  la valeur en edx
		mov BYTE PTR [esi+7],dl							;; placer l'octet en dl  l'adresse en esi
		mul ebx										;; multiplier eax par 10
Label10:	add dl,"0"									;; ajouter le caractre de base "0"  la valeur en edx
		mov BYTE PTR [esi+8],dl							;; placer l'octet en dl  l'adresse en esi
Label11:	mul ebx										;; multiplier eax par 10
		add dx,"0"									;; ajouter le caractre de base "0"  la valeur en dx
		mov WORD PTR [esi+9],dx							;; placer l'octet en dl et le 0 final en dh  l'adresse en esi
		lea eax,[esi+10]								;; copier esi (l'adresse en cours+10) dans eax

		pop esi										;; dsempiler esi
		pop edx										;; dsempiler edx
		pop ecx										;; dsempiler ecx
		pop ebx										;; dsempiler ebx
		sub eax,esi									;; pour obtenir la taille de la chane cre dans eax
	ret												;; retourner (sortir de la procdure)
;ALIGN 4
CasMoinsUn:
		mov DWORD PTR [esi],"4924"						;; ) placer directement la valeur correspondante
		mov DWORD PTR [esi+4],"2769"						;; )
		mov WORD PTR [esi+8],"59"						;; )
		mov BYTE PTR [esi+10],0							;; et le zro final
		mov eax,10									;; pour obtenir la taille de la chane cre dans eax

		pop esi										;; dsempiler esi
		pop edx										;; dsempiler edx
		pop ecx										;; dsempiler ecx
		pop ebx										;; dsempiler ebx
	ret												;; retourner (sortir de la procdure)
;ALIGN 4
CasZero:
		mov WORD PTR [esi],"0"							;; placer "0",0
		inc eax										;; pour obtenir la taille de la chane cre dans eax
	ret												;; retourner (sortir de la procdure)
ConvertirValeurEnChaineDecimale ENDP


ALIGN 16
;
; copier une chane de caractres dans une autre chane
; note : videment, les tailles doivent tre valides pour les chanes source et destination
;
; Syntaxe :
; mov esi,{adresse (offset) de la chane source}
; mov edi,{adresse (offset) dans la chane de destination}
; syntaxe : CopierChaineAscii
;
; Retourne :
; eax = nombre de caractres copis (c-a-d la taille ajoute  la chane)
;
CopierChaineAscii PROC
		push ecx									;; empiler ecx
		push edx									;; empiler edx
		push esi									;; empiler esi

		sub edi,esi								;; soustraire l'adresse 1  l'adresse 2 (pour n'avoir que l'cart en edi)
		jmp Label2								;; aller Label2
;		nop										;; ) aucun alignement ncessaire pour un meilleur rendement
;		nop										;; )
;		nop										;; )
;		nop										;; )
;		nop										;; )
;		nop										;; )
;		nop										;; )
;		nop										;; )
Label1:	mov DWORD PTR [esi+edi],edx					;; placer cette valeur  l'adresse de destination
		add esi,4									;; incrmenter esi (l'adresse) de 4 (un dword)
Label2:	mov edx,DWORD PTR [esi]						;; placer le motlong de l'adresse actuelle dans edx
		lea ecx,[edx-01010101h]						;; placer edx (- 1  chaque octets du motlong) dans ecx
		mov eax,edx								;; copier la valeur dans eax
		xor eax,ecx								;; inverser les bits en eax, en utilisant le rsultat obtenu dans ecx comme masque
		and eax,80808080h							;; ne conserver que le bit de poids fort de chaques octets en eax
		jz Label1									;; si le filtrage est gal  0, aller Label1
		and eax,ecx								;; ne conserver en eax que les bits positionns en ecx
		jz Label1									;; si le filtrage est gal  0, aller Label1
		mov [esi+edi],dl							;; placer dl  l'adresse en edi+esi
		test dl,dl								;; fixer les flags de dl
		jz Label5									;; si c'est gal  0, aller Label5
		mov [esi+edi+1],dh							;; placer dh  l'adresse en edi+esi+1
		test dh,dh								;; fixer les flags de dh
		jz Label4									;; si c'est gal  0, aller Label4
		shr edx,16								;; dcaler edx  droite de 16 bits
		mov [esi+edi+2],dl							;; placer dl  l'adresse en edi+esi+2
		test dl,dl								;; fixer les flags de dl
		jz Label3									;; si c'est gal  0, aller Label3
		mov [esi+edi+3],dh							;; placer dh  l'adresse en edi+esi+3
		inc esi									;; ajouter 1  esi ) (optionnel) pour obtenir la taille de la chaine dans eax
Label3:	inc esi									;; ajouter 1  esi )
Label4:	inc esi									;; ajouter 1  esi )
Label5:	mov eax,esi								;; placer esi dans eax

		pop esi									;; dsempiler esi
		pop edx									;; dsempiler edx
		pop ecx									;; dsempiler ecx
		sub eax,esi								;; pour obtenir la taille de la chaine dans eax
		add edi,esi								;; pour restaurer edi
	ret												;; retourner (sortir de la procdure)
CopierChaineAscii ENDP


ALIGN 16
;
; fusionner des chaines de caractres
; note : c'est plus rapide si edi pointe exactement  l'emplacement o on ajoutera la chane, mais dans ce cas l,
; eax retournera uniquement la taille de la chane ajoute
;
; Syntaxe :
; mov esi,{adresse (offset) de la chane source que l'on ajoutera}
; mov edi,{adresse (offset) de/dans la chane de destination}
; call FusionnerChainesAscii
;
; Retourne :
; eax = la taille de la chaine ou la taille de la chaine ajoute (suivant que edi contenait l'adresse de dpart ou l'adresse de fin de la chaine de destination)
;
FusionnerChainesAscii PROC
		push ecx										;; empiler ecx
		push edx										;; empiler edx
		push edi										;; empiler edi

; premirement on va trouver le zro final de la chaine de destination
Label1:	mov eax,DWORD PTR [edi]							;; placer le dword de l'adresse actuelle dans eax
		add edi,4										;; incrmenter edi (l'adresse) de 4 (un dword)
		lea edx,[eax-01010101h]							;; placer eax (- 1  chaque octets du dword) dans edx
		xor eax,edx									;; inverser les bits en eax, en utilisant le rsultat obtenu dans edx comme masque
		and eax,80808080h								;; ne conserver que le bit de poids fort de chaques octets en eax
		jz Label1										;; si le filtrage est gal  0, aller Label1
		and eax,edx									;; ne conserver en eax que les bits positionns en edx
		jz Label1										;; si le filtrage est gal  0, aller Label1
		bsf edx,eax									;; sinon, scanner le premier bit arm en eax, en commencant par la droite. et placer le numro du bit en edx
		sub edx,4										;; soustraire 4 au numro du bit obtenu
		shr edx,3										;; dcaler edx  droite de 3 bits pour obtenir le nombre d'octets
		lea edi,[edi+edx-4]								;; placer edi + edx - 1 dword dans edi
; et on effectue la copie
		xor eax,eax									;; effacer eax (notre compteur d'octets)
		jmp Label3									;; aller Label3
;		nop											;; ) aucun alignement ncessaire pour un meilleur rendement
;		nop											;; )
;		nop											;; )
;		nop											;; )
;		nop											;; )
;		nop											;; )
;		nop											;; )
;		nop											;; )
Label2:	add ecx,01010101h								;; ajouter ce que l'on a enlev prcedemment pour retrouver la valeur de edx
		mov DWORD PTR [edi+eax],ecx						;; placer cette valeur  l'adresse de destination
		add eax,4										;; incrmenter esi (l'adresse) de 4 (un dword)
Label3:	mov edx,DWORD PTR [esi+eax]						;; placer le dword de l'adresse actuelle dans edx
		lea ecx,[edx-01010101h]							;; placer edx (- 1  chaque octets du dword) dans ecx
		xor edx,ecx									;; inverser les bits en edx, en utilisant le rsultat obtenu dans ecx comme masque
		and edx,80808080h								;; ne conserver que le bit de poids fort de chaques octets en edx
		jz Label2										;; si le filtrage est gal  0, aller Label2
		and edx,ecx									;; ne conserver en edx que les bits positionns en ecx
		jz Label2										;; si le filtrage est gal  0, aller Label2
		add ecx,01010101h								;; ajouter ce que l'on a enlev prcedemment pour retrouver la valeur de edx
		mov [edi+eax],cl								;; placer cl  l'adresse en edi+eax
		test cl,cl									;; fixer les flags de cl
		jz Label6										;; si c'est gal  0, aller Label6
		mov [edi+eax+1],ch								;; placer ch  l'adresse en edi+eax+1
		test ch,ch									;; fixer les flags de ch
		jz Label5										;; si c'est gal  0, aller Label5
		shr ecx,16									;; dcaler ecx  droite de 16 bits
		mov [edi+eax+2],cl								;; placer cl  l'adresse en edi+eax+2
		test cl,cl									;; fixer les flags de cl
		jz Label4										;; si c'est gal  0, aller Label4
		mov [edi+eax+3],ch								;; placer ch  l'adresse en edi+eax+3
		inc eax										;; ajouter 1  eax ) (optionnel) pour obtenir la taille de la chaine dans eax
Label4:	inc eax										;; ajouter 1  eax )
Label5:	inc eax										;; ajouter 1  eax )
Label6:	add eax,edi									;; ajouter edi  eax

		pop edi										;; dsempiler edi
		pop edx										;; dsempiler edx
		pop ecx										;; dsempiler ecx
		sub eax,edi									;; et on soustrait l'adresse originelle  eax pour obtenir la nouvelle taille de la chane
	ret												;; retourner (sortir de la procdure)
FusionnerChainesAscii ENDP


ALIGN 16
;
; fusionner des chaines de caractres
;
; Rceptionne :
; edi = adresse de la chane de destination
; le nombre de chaines  ajouter (1 au minimum) (qui servira de compteur)
; et les adresses des diffrentes chanes sources que l'on ajoutera
;
; Retourne :
; eax = la taille de la nouvelle chaine obtenue
;
; syntaxe : FusionnerMultiplesChainesAscii [nombre de chaines  ajouter],[adresse de la chane source1],[adresse de la chane source2],...
;
FusionnerMultiplesChainesAscii PROC C _Compteur_:DWORD,_Adresse_Sources_:VARARG
		push ebx										;; empiler ebx
		push ecx										;; empiler ecx
		push edx										;; empiler edx
		push esi										;; empiler esi
		push edi										;; empiler edi

		lea esi,_Adresse_Sources_						;; placer l'adresse de la premire chane source dans esi
		nop											;; alignement ncessaire pour un meilleur rendement

; premirement on va trouver le zro final de la chaine de destination
Label1:	mov eax,DWORD PTR [edi]							;; placer le motlong de l'adresse actuelle dans eax
		add edi,4										;; incrmenter edi (l'adresse) de 4 (un dword)
		lea edx,[eax-01010101h]							;; placer eax (- 1  chaque octets du motlong) dans edx
		xor eax,edx									;; inverser les bits en eax, en utilisant le rsultat obtenu dans edx comme masque
		and eax,80808080h								;; ne conserver que le bit de poids fort de chaques octets en eax
		jz Label1										;; si le filtrage est gal  0, aller Label1
		and eax,edx									;; ne conserver en eax que les bits positionns en edx
		jz Label1										;; si le filtrage est gal  0, aller Label1
		bsf edx,eax									;; sinon, scanner le premier bit arm en eax, en commencant par la droite. et placer le numro du bit en edx
		sub edx,4										;; soustraire 4 au numro du bit obtenu
		shr edx,3										;; dcaler edx  droite de 3 bits pour obtenir le nombre d'octets
		lea edi,[edi+edx-4]								;; placer edi + edx - 1 dword dans edi

; ensuite on va prparer nos copie de chaines
		xor ebx,ebx									;; effacer ebx (notre compteur de chanes)
Label2:	push ebx										;; empiler ebx (parcequ'on  plus de registres libres)
		mov ebx,[4*ebx+esi]								;; placer l'adresse de la nouvelle source dans ebx
		xor eax,eax									;; effacer eax (notre compteur d'octets)
		jmp Label4									;; aller Label4

; et on effectue nos copies
Label3:	add ecx,01010101h								;; ajouter ce que l'on a enlev prcedemment pour retrouver la valeur de eax
		mov DWORD PTR [edi+eax],ecx						;; placer cette valeur  l'adresse de destination
		add eax,4										;; incrmenter eax (notre compteur d'octets) de 4 (un dword)
Label4:	mov edx,DWORD PTR [ebx+eax]						;; placer le motlong de l'adresse actuelle dans edx
		lea ecx,[edx-01010101h]							;; placer eax (- 1  chaque octets du motlong) dans ecx
		xor edx,ecx									;; inverser les bits en eax, en utilisant le rsultat obtenu dans ecx comme masque
		and edx,80808080h								;; ne conserver que le bit de poids fort de chaques octets en eax
		jz Label3										;; si le filtrage est gal  0, aller Label3
		and edx,ecx									;; ne conserver en edx que les bits positionns en ecx
		jz Label3										;; si le filtrage est gal  0, aller Label3
		add ecx,01010101h								;; ajouter ce que l'on a enlev prcedemment pour retrouver la valeur de edx
		mov [edi+eax],cl								;; placer cl  l'adresse en edi+eax
		test cl,cl									;; fixer les flags de cl
		jz Label7										;; si c'est gal  0, aller Label7
		mov [edi+eax+1],ch								;; placer ch  l'adresse en edi+eax+1
		test ch,ch									;; fixer les flags de ch
		jz Label6										;; si c'est gal  0, aller Label6
		shr ecx,16									;; dcaler ecx  droite de 16 bits
		mov [edi+eax+2],cl								;; placer cl  l'adresse en edi+eax+2
		test cl,cl									;; fixer les flags de cl
		jz Label5										;; si c'est gal  0, aller Label5
		mov [edi+eax+3],ch								;; placer ch  l'adresse en edi+eax+3
		inc eax										;; ajouter 1  eax
Label5:	inc eax										;; ajouter 1  eax
Label6:	inc eax										;; ajouter 1  eax
Label7:	lea edi,[edi+eax]								;; ajouter eax+1  edi

; l on va tester s'il y a une autre source  ajouter
		pop ebx										;; dsempiler ebx (pour r-obtenir notre compteur de sources)
		inc ebx										;; ajouter 1 au nombre de source
		cmp ebx,_Compteur_								;; et le comparer  Compteur
		jne Label2									;; si c'est diffrent, aller Label2

; ici, on calcul la nouvelle taille de la chane...
		mov eax,edi									;; placer edi dans eax

		pop edi										;; dsempiler edi
		sub eax,edi									;; et soustraire AdresseDestination  eax pour obtenir la nouvelle taille de la chane
		pop esi										;; dsempiler esi
		pop edx										;; dsempiler edx
		pop ecx										;; dsempiler ecx
		pop ebx										;; dsempiler ebx
	ret												;; retourner (sortir de la procdure)
FusionnerMultiplesChainesAscii ENDP


ALIGN 16
;
; obtenir la taille relle d'une chane de caractres
;
; Syntaxe :
; mov esi,{adresse (offset) de la chane dont on veut obtenir la taille}
; call ObtenirTailleChaineAscii
;
; Retourne :
; eax = taille relle de la chane
;
ObtenirTailleChaineAscii PROC
		push edx										;; empiler edx
		push esi										;; empiler esi

;		nop											;; ) aucun alignement ncessaire pour un meilleur rendement
;		nop											;; )
;		nop											;; )
;		nop											;; )
;		nop											;; )
;		nop											;; )
;		nop											;; )
;		nop											;; )
Label1:	mov eax,DWORD PTR [esi]							;; placer le dword de l'adresse actuelle dans eax
		add esi,DWORD									;; incrmenter esi (l'adresse) de 4 (un dword)
		lea edx,[eax-01010101h]							;; placer eax (- 1  chaque octets du dword) dans edx
		xor eax,edx									;; inverser les bits en eax, en utilisant le rsultat obtenu dans edx comme masque
		and eax,80808080h								;; ne conserver que le bit de poids fort de chaques octets en eax
		jz Label1										;; si le filtrage est gal  0, aller Label1
		and eax,edx									;; ne conserver en eax que les bits positionns en edx
		jz Label1										;; si le filtrage est gal  0, aller Label1
		bsf eax,eax									;; sinon, scanner le premier bit arm en eax, en commencant par la droite. et placer le numro du bit en eax
		sub eax,4										;; soustraire 4 au numro du bit obtenu
		shr eax,3										;; dcaler eax  droite de 3 bits pour obtenir le nombre d'octets
		lea eax,[esi+eax-DWORD]							;; placer esi + eax - 1 dword dans eax

		pop esi										;; dsempiler esi
		pop edx										;; dsempiler edx
		sub eax,esi									;; et soustraire l'adresse originelle au rsultat obtenu

		push edx										;; empiler edx
		push esi										;; empiler esi
	ret												;; retourner (sortir de la procdure)
ObtenirTailleChaineAscii ENDP


